/**
 * EpicMusicPlayer Playlistgenerator
 *
 * @author yess
 */

package app;

import com.ice.jni.registry.Registry;
import com.ice.jni.registry.RegistryKey;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * 
 */
public class Config {

    private File musicDir;
    private String fileName;
    private File wowDirectory;
    private String loadfrom = "";
    private String saveto = "";
    private boolean duplicate = false;
    private boolean saved = true;
    
    private boolean copyfiles = true;
    private Junction jLink;
    
    public Config() {
        fileName = "emp.config";
    }

    
    public void load() throws Exception {
                
        //try load saved config
        try {
        File conffile = new File(fileName);
        String a = conffile.getAbsolutePath();
        BufferedReader in = new BufferedReader(new FileReader(fileName));
        String line = in.readLine();
        
        while (line != null) {
            if (line.contains("musicfolder")) {
                String[] path = line.split("=");
                //System.out.println(path[1]);
                setMusicFloder(new File(path[1]));
            }
            
            if (line.contains("wowfolder")) {
                String[] path = line.split("=");
                //System.out.println(path[1]);
                setWoWDir(new File(path[1]));
            }
            if (line.contains("loadfrom")) {
                String[] path = line.split("=");
                //System.out.println(path[1]);
                setLoadfrom(path[1]);
            }
            if (line.contains("saveto")) {
                String[] path = line.split("=");
                //System.out.println(path[1]);
                setSaveto(path[1]);
            }
            if (line.contains("dublicate")) {
                String[] dub = line.split("=");
                //System.out.println(dub[1]);
                if(dub[1].equalsIgnoreCase("true"))
                {
                    setDuplicate(true);
                }
            }
            if (line.contains("copyfiles")) {
                String[] cop = line.split("=");
                //System.out.println(dub[1]);
                if(cop[1].equalsIgnoreCase("false"))
                {
                    setCopyfiles(false);
                }
            }
            line = in.readLine();
            
        }
        } catch (FileNotFoundException e1){
            e1.printStackTrace();
            throw new Exception("Configuration file not found.");  
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Error loading config.");  
        }
    }

    public void save() throws IOException {
        BufferedWriter out = new BufferedWriter(new FileWriter(fileName, false));
        out.write("musicfolder=" + this.musicDir);
        out.newLine();
        out.write("wowfolder=" + getWoWDir());
        out.newLine();
        out.write("loadfrom=" + getLoadfrom());
        out.newLine();
        out.write("saveto=" + getSaveto());
        out.newLine();
        out.write("dublicate=" + isDuplicate());
        out.newLine();
        out.write("copyfiles=" + isCopyfiles());
        out.newLine();

        out.close();
    }

    public void setWowfolder(File wowfolder) throws FileNotFoundException {
        //check if in wow folder
        File addondir = findWowDir(wowfolder);
        if(addondir==null){
            throw new FileNotFoundException("wow.exe not found in selected World of Warcraft folder");
        }
        else
        {
            this.wowDirectory = addondir;
        }   
    }

    /**
     * Set music folder without check if in wow folder
     * @param file
     */
    private void setMusicFloder(File file) {
        if(file == null)
            throw new IllegalArgumentException("MusicFloder not set.");
        if(file.getPath().equals("") || file.getPath().equalsIgnoreCase("null"))
            throw new IllegalArgumentException("MusicFloder not set.");
        
        this.musicDir = file;
    }

    private void setWoWDir(File file)
    {
        if(file == null)
            throw new IllegalArgumentException("WoW folder not set.");
        if(file.getPath().equals("") || file.getPath().equalsIgnoreCase("null"))
            throw new IllegalArgumentException("WoW folder not set.");
        
        wowDirectory = file;
    }
    
    public File getWoWDir()
    {
        return wowDirectory;
    }
    
    public File getWoWFolderRegistry()
    {
        File wowfolder = null;
        //try read path from registry
            try {
                Registry reg = new Registry();
                RegistryKey topKey = reg.getTopLevelKey("HKEY_LOCAL_MACHINE");
                RegistryKey subKey = reg.openSubkey(topKey, "SOFTWARE\\Blizzard Entertainment\\World of Warcraft", 0);
                //System.out.println(subKey.getStringValue("InstallPath"));
                wowfolder = new File(subKey.getStringValue("InstallPath"));
            } catch (UnsatisfiedLinkError linkerror) {
                System.out.print("Could not read the WoW install path from registry.");
            } catch (Exception e1) {
                System.out.print("Could not read the WoW install path from registry.");
            }
        return wowfolder;
    }
    
    public File createDefaultMusicDir(File wowfolder) throws FileNotFoundException
    {
        File musicdir = new File(wowfolder.getPath() +"/MyMusic");
        if(musicdir.exists())
        {
            return musicdir;
        }
        else if(musicdir.mkdir())
        {
            return musicdir;
        }
        else
        {
            throw new FileNotFoundException("Could not create Folder at "+musicdir.getPath());
        }
    }
    
    public void setMusicDir(File musicdir) throws FileNotFoundException {
        if (!musicdir.exists()) {
            throw new FileNotFoundException("Directory not found.\n" + musicdir.getPath());
        }

        //check if in wow folder
        File addondir = findWowDir(musicdir);
        if(addondir==null){
            throw new FileNotFoundException("Music folder is not in World of Warcraft folder");
        }
        else
        {
            //set music folder
            System.out.println("wowdir="+addondir.getPath());
            this.musicDir = musicdir;
            this.wowDirectory = addondir;
        }   
        
    }
    
    private File findWowDir(File musicdir)
    {
        
        while(musicdir !=null)
        {    
            System.out.println("cheking="+musicdir.getPath());
            String[] children = musicdir.list();
            for(int i=0; i<children.length;i++)
            {
                if(children[i].equalsIgnoreCase("WTF"))
                {
                    return musicdir; //wow folder found
                }
            }
            musicdir = musicdir.getParentFile();
        }
        return null;
    }
    
    public File getPlaylistFile(String accountname){
        return new File(getWoWDir().getPath() + "/WTF/Account/" + accountname + "/SavedVariables/EpicMusicPlayer.lua");
    }

    public List getAccunts(File wowdir) throws FileNotFoundException {

        File accountsdir = new File(wowdir.getPath() + "/WTF/Account/");

        if (!accountsdir.exists()) {
            throw new FileNotFoundException("Account directory not found in: \n" + accountsdir.getPath());
        }

        ArrayList list = new ArrayList();
        String[] children = accountsdir.list();
        for (int i = 0; i < children.length; i++) {
            //System.out.println(children[i]);
            list.add(children[i]);
        }


        return list;
    }

    public File getMusicDir() {
        return musicDir;
    }

    public boolean isSaved() {
        return saved;
    }

    public void setSaved(boolean saved) {
        this.saved = saved;
    }

    

    public String getLoadfrom() {
        return loadfrom;
    }

    public void setLoadfrom(String loadfrom) {
        if(loadfrom == null)
            throw new IllegalArgumentException("Account not set.");
        if(loadfrom.equals("") || loadfrom.equalsIgnoreCase("null"))
            throw new IllegalArgumentException("Account not set.");
        
        this.loadfrom = loadfrom;
    }

    public String getSaveto() {
        return saveto;
    }

    public void setSaveto(String saveto) {
        if(saveto == null)
            throw new IllegalArgumentException("Account not set.");
        if(saveto.equals("") || saveto.equalsIgnoreCase("null"))
            throw new IllegalArgumentException("Account not set.");
        
        this.saveto = saveto;
    }

    public boolean isDuplicate() {
        return duplicate;
    }

    public void setDuplicate(boolean duplicate) {
        this.duplicate = duplicate;
    }

    public boolean isCopyfiles() {
        return copyfiles;
    }

    public void setCopyfiles(boolean copyfiles) throws Exception {
        if(!copyfiles){
            this.setJLink(new Junction());
        }
        this.copyfiles = copyfiles;
    }

    public Junction getJLink() {
        return jLink;
    }

    public void setJLink(Junction jLink) {
        this.jLink = jLink;
    }

}
